"""Defaults settings"""

DEFAULT_CONTEXT_SETTINGS = {
    "max_content_width": 140,
    "terminal_width": 140,
    "help_option_names": ["-h", "--help"],
    # 'help_headers_color': 'yellow',
    # 'help_options_color': 'green',
}
